clear;
%% Numerical Solution for Fractional Diffusion-Reaction Equation 
%% with g(u)= u(1-u), Neumann BC: u'(0,t) = u'(L,t)=0

%% Discretise in space and time
L = 3; % spatial domain [0,L]
N = 1000; % number of divisions in space
h = L/N; % stepsize in space
x = linspace(0,L,N+1); % N+1 nodes on [0,L]
D = 0.01; % diffusion coefficent
alpha = 1.8; % fractional power
T = 20; % final time
n = 1000; % number of time steps
tau = T/n; % time step
u0 = exp(-100*x.^2)'; % initial condition
 
%% Matrix representation of Laplacian (-Delta)
A = gallery('tridiag', N+1);
A(1,1) = 1; A(1,2) = -1; % zero Neumann BC
A(N+1,N) = -1; A(N+1,N+1) = 1; % zero Neumann BC
A = 1/h^2*full(A);
[P,Lam] = eig(A); % diagonalise A such that A=P*Lam*P^T
lambda = diag(Lam); % eigenvalues
scaling = (1+D*tau*lambda.^(alpha/2)); % To compute un using diagonalisation

%% using discrete cosine tranform
% lam = 4/h^2 * sin(pi*(0:N)'/2/(N+1)).^2; % formula for eigenvalues under Zero Neumann BC.
% scaling2 = 1+D*tau*lam.^(alpha/2); % computing un using formula for e.values

%% computing un
un = u0; 
figure; 
plot(x, u0', 'LineWidth', 2); xlabel('x');ylabel('u(x,t)');
title({'Numerical solution of Fractional Diffusion Reaction Equation', 'BC: zero Neumann, IC: exp(-100*x^2)'});

for j = 1:n % timesteping
    vn = un + tau*un.*(1-un);
    un = P*((P'*vn)./scaling); % using diagonalisation
    %un = idct(dct(vn)./scaling2); %using discrete cosine transform
    if mod(j,n/10)==0 % plot every 100 time steps
        hold on
        handle = plot(x, un', 'LineWidth', 2);
        legend(handle,['T = ',num2str(j*tau),' with \alpha = ',num2str(alpha)]);
        ylim([0 1]); 
        shg;
    end
end
